@@ -19,9 +19,13 @@
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
+#include "config.h"
+
 #define _XOPEN_SOURCE 600
 #include "dconf-engine.h"
 
+#include "../common/dconf-error.h"
+#include "../common/dconf-paths.h"
 #include "../gvdb/gvdb-reader.h"
 #include <string.h>
 #include <stdlib.h>
@@ -221,8 +225,9 @@ dconf_engine_unlock_queues (DConfEngine *engine)
 }
 
 DConfEngine *
-dconf_engine_new (gpointer       user_data,
-                  GDestroyNotify free_func)
+dconf_engine_new (const gchar    *profile,
+                  gpointer        user_data,
+                  GDestroyNotify  free_func)
 {
   DConfEngine *engine;
 
@@ -235,7 +240,7 @@ dconf_engine_new (gpointer       user_data,
   g_mutex_init (&engine->queue_lock);
   g_cond_init (&engine->queue_cond);
 
-  engine->sources = dconf_engine_profile_open (NULL, &engine->n_sources);
+  engine->sources = dconf_engine_profile_open (profile, &engine->n_sources);
 
   g_mutex_lock (&dconf_engine_global_lock);
   dconf_engine_global_list = g_slist_prepend (dconf_engine_global_list, engine);
@@ -564,6 +569,8 @@ dconf_engine_read_user_value (DConfEngine *engine,
   if (engine->n_sources == 0 || !engine->sources[0]->writable)
     return NULL;
 
+  dconf_engine_acquire_sources (engine);
+
   /* First check read-through */
   if (read_through)
     found_key = dconf_engine_find_key_in_queue (read_through, key, &value);
@@ -702,7 +709,10 @@ dconf_engine_call_handle_new (DConfEngine                   *engine,
 const GVariantType *
 dconf_engine_call_handle_get_expected_type (DConfEngineCallHandle *handle)
 {
-  return handle->expected_reply;
+  if (handle)
+    return handle->expected_reply;
+  else
+    return NULL;
 }
 
 void
@@ -777,7 +787,7 @@ dconf_engine_watch_established (DConfEngine  *engine,
        * We don't know what changed, so we can just say that potentially
        * everything changed.  This case is very rare, anyway...
        */
-      dconf_engine_change_notify (engine, "/", changes, NULL, NULL, engine->user_data);
+      dconf_engine_change_notify (engine, "/", changes, NULL, FALSE, NULL, engine->user_data);
     }
 
   dconf_engine_call_handle_free (handle);
@@ -913,7 +923,7 @@ dconf_engine_emit_changes (DConfEngine    *engine,
   const gchar * const *changes;
 
   if (dconf_changeset_describe (changeset, &prefix, &changes, NULL))
-    dconf_engine_change_notify (engine, prefix, changes, NULL, origin_tag, engine->user_data);
+    dconf_engine_change_notify (engine, prefix, changes, NULL, FALSE, origin_tag, engine->user_data);
 }
 
 static void
@@ -1172,10 +1182,29 @@ dconf_engine_change_sync (DConfEngine     *engine,
   return TRUE;
 }
 
+static gboolean
+dconf_engine_is_interested_in_signal (DConfEngine *engine,
+                                      GBusType     bus_type,
+                                      const gchar *sender,
+                                      const gchar *path)
+{
+  gint i;
+
+  for (i = 0; i < engine->n_sources; i++)
+    {
+      DConfEngineSource *source = engine->sources[i];
+
+      if (source->bus_type == bus_type && g_str_equal (source->object_path, path))
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 void
 dconf_engine_handle_dbus_signal (GBusType     type,
                                  const gchar *sender,
-                                 const gchar *path,
+                                 const gchar *object_path,
                                  const gchar *member,
                                  GVariant    *body)
 {
@@ -1191,6 +1220,36 @@ dconf_engine_handle_dbus_signal (GBusType     type,
 
       g_variant_get (body, "(&s^a&s&s)", &prefix, &changes, &tag);
 
+      /* Reject junk */
+      if (changes[0] == NULL)
+        /* No changes?  Do nothing. */
+        goto junk;
+
+      if (dconf_is_key (prefix, NULL))
+        {
+          /* If the prefix is a key then the changes must be ['']. */
+          if (changes[0][0] || changes[1])
+            goto junk;
+        }
+      else if (dconf_is_dir (prefix, NULL))
+        {
+          /* If the prefix is a dir then we can have changes within that
+           * dir, but they must be rel paths.
+           *
+           *   ie:
+           *
+           *  ('/a/', ['b', 'c/']) == ['/a/b', '/a/c/']
+           */
+          gint i;
+
+          for (i = 0; changes[i]; i++)
+            if (!dconf_is_rel_path (changes[i], NULL))
+              goto junk;
+        }
+      else
+        /* Not a key or a dir? */
+        goto junk;
+
       g_mutex_lock (&dconf_engine_global_lock);
       engines = g_slist_copy_deep (dconf_engine_global_list, (GCopyFunc) dconf_engine_ref, NULL);
       g_mutex_unlock (&dconf_engine_global_lock);
@@ -1206,22 +1265,48 @@ dconf_engine_handle_dbus_signal (GBusType     type,
            * Check last_handled to determine if we should ignore it.
            */
           if (!engine->last_handled || !g_str_equal (engine->last_handled, tag))
-            dconf_engine_change_notify (engine, prefix, changes, tag, NULL, engine->user_data);
+            if (dconf_engine_is_interested_in_signal (engine, type, sender, object_path))
+              dconf_engine_change_notify (engine, prefix, changes, tag, FALSE, NULL, engine->user_data);
 
           engines = g_slist_delete_link (engines, engines);
 
           dconf_engine_unref (engine);
         }
 
+junk:
       g_free (changes);
     }
 
   else if (g_str_equal (member, "WritabilityNotify"))
     {
+      const gchar *empty_str_list[] = { "", NULL };
+      const gchar *path;
+      GSList *engines;
+
       if (!g_variant_is_of_type (body, G_VARIANT_TYPE ("(s)")))
         return;
 
-      g_warning ("Need to handle writability changes"); /* XXX */
+      g_variant_get (body, "(&s)", &path);
+
+      /* Rejecting junk here is relatively straightforward */
+      if (!dconf_is_path (path, NULL))
+        return;
+
+      g_mutex_lock (&dconf_engine_global_lock);
+      engines = g_slist_copy_deep (dconf_engine_global_list, (GCopyFunc) dconf_engine_ref, NULL);
+      g_mutex_unlock (&dconf_engine_global_lock);
+
+      while (engines)
+        {
+          DConfEngine *engine = engines->data;
+
+          if (dconf_engine_is_interested_in_signal (engine, type, sender, object_path))
+            dconf_engine_change_notify (engine, path, empty_str_list, "", TRUE, NULL, engine->user_data);
+
+          engines = g_slist_delete_link (engines, engines);
+
+          dconf_engine_unref (engine);
+        }
     }
 }
 
