@@ -19,6 +19,8 @@
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
+#include "config.h"
+
 #define G_SETTINGS_ENABLE_BACKEND
 #include <gio/gsettingsbackend.h>
 #include "../engine/dconf-engine.h"
@@ -181,7 +183,7 @@ dconf_settings_backend_init (DConfSettingsBackend *dcsb)
 
   weak_ref = g_slice_new (GWeakRef);
   g_weak_ref_init (weak_ref, dcsb);
-  dcsb->engine = dconf_engine_new (weak_ref, dconf_settings_backend_free_weak_ref);
+  dcsb->engine = dconf_engine_new (NULL, weak_ref, dconf_settings_backend_free_weak_ref);
 }
 
 static void
@@ -239,6 +241,7 @@ dconf_engine_change_notify (DConfEngine         *engine,
                             const gchar         *prefix,
                             const gchar * const *changes,
                             const gchar         *tag,
+                            gboolean             is_writability,
                             gpointer             origin_tag,
                             gpointer             user_data)
 {
@@ -253,6 +256,21 @@ dconf_engine_change_notify (DConfEngine         *engine,
   if (changes[0] == NULL)
     return;
 
+  if (is_writability)
+    {
+      /* We know that the engine does it this way... */
+      g_assert (changes[0][0] == '\0' && changes[1] == NULL);
+
+      if (g_str_has_suffix (prefix, "/"))
+        g_settings_backend_path_writable_changed (G_SETTINGS_BACKEND (dcsb), prefix);
+      else
+        g_settings_backend_writable_changed (G_SETTINGS_BACKEND (dcsb), prefix);
+    }
+
+  /* We send the normal change notification even in the event that this
+   * was a writability notification because adding/removing a lock could
+   * impact the value that gets read.
+   */
   if (changes[1] == NULL)
     {
       if (g_str_has_suffix (prefix, "/"))
